=== modified file 'plugin/auth_schema/auth_schema.cc'
--- plugin/auth_schema/auth_schema.cc	2011-10-07 03:00:33 +0000
+++ plugin/auth_schema/auth_schema.cc	2011-10-08 19:47:11 +0000
@@ -36,10 +36,23 @@
 {
 }
 
-bool AuthSchema::setTable(const char *table)
+bool AuthSchema::setTable(const string &table)
 {
+  if (table.empty())
+  {
+    errmsg_printf(error::ERROR, _("auth_schema table cannot be an empty string"));
+    return true;  // error
+  }
+
+  if (table.find(" ") != string::npos)
+  {
+    errmsg_printf(error::ERROR, _("auth_schema table cannot contain spaces"));
+    return true;  // error
+  }
+
   sysvar_table= table;
-  return false;
+
+  return false;  // success
 }
 
 bool AuthSchema::verifyMySQLPassword(const string &real_password,
@@ -91,13 +104,14 @@
     return false;
 
   // Anonymous users are not allowed.
-  string user= sctx.username();
+  string user= escapeString(sctx.username());
   if (user.empty())
     return false;
 
   // Create an internal session for ourself the first time we're called.
   // I don't know why but doing this in the constructor crashes Drizzle
-  if (not _session) {
+  if (not _session)
+  {
     _session= Session::make_shared(plugin::Listen::getNullClient(), catalog::local());
     identifier::user::mptr user_id= identifier::User::make_shared();
     user_id->setUser("auth_schema");
@@ -106,9 +120,7 @@
 
   // Create an execute a SQL statement to select the user from the auth table.
   // Execute wraps the SQL to run within a transaction.
-  string sql= "SELECT password FROM " + sysvar_table +
-              " WHERE user='" + user + "'"
-              " LIMIT 1;";
+  string sql= "SELECT password FROM " + sysvar_table + " WHERE user='" + user + "' LIMIT 1;";
   Execute execute(*(_session.get()), true);
   sql::ResultSet result_set(1);
   execute.run(sql, result_set);
@@ -117,8 +129,8 @@
   if ((err != EE_OK) && (err != ER_EMPTY_QUERY))
   {
     errmsg_printf(error::ERROR,
-      _("Error querying authentication schema: %s (error code %d)"),
-      exception.getErrorMessage().c_str(), exception.getErrorCode());
+      _("Error querying authentication schema: %s (error code %d.  Query: %s"),
+      exception.getErrorMessage().c_str(), exception.getErrorCode(), sql.c_str());
     return false;
   }
 
@@ -138,5 +150,47 @@
   return false;
 }
 
+string AuthSchema::escapeString(const string &input)
+{
+  return input;
+  if (input.empty())
+    return input;
+
+  const char *pos= input.c_str();
+  const char *end= input.c_str()+input.length();
+  string res;
+
+  for (; pos != end ; pos++)
+  {
+    switch (*pos) {
+    case 0:
+      res.push_back('\\');
+      res.push_back('0');
+      break;
+    case '\n':
+      res.push_back('\\');
+      res.push_back('n');
+      break;
+    case '\r':
+      res.push_back('\\');
+      res.push_back('r');
+      break;
+    case '\\':
+      res.push_back('\\');
+      res.push_back('\\');
+      break;
+    case '\'':
+      res.push_back('\\');
+      res.push_back('\'');
+      break;
+    default:
+      res.push_back(*pos);
+      break;
+    }
+  }
+
+  return res;
+}
+
 } /* end namespace drizzle_plugin::auth_schema */
 } /* end namespace drizzle_plugin */

=== modified file 'plugin/auth_schema/auth_schema.h'
--- plugin/auth_schema/auth_schema.h	2011-10-07 03:00:33 +0000
+++ plugin/auth_schema/auth_schema.h	2011-10-08 19:47:11 +0000
@@ -34,13 +34,14 @@
 
   /**
    * @brief
-   *   Set/change the authentication table.
+   *   Set the authentication table.
    *
    * @param[in] table Schema-qualified table name.
    *
-   * @returns false Always returns false currently because table isn't checked.
+   * @retval false Success, new auth table set
+   * @retval true  Failure, auth table not changed
    */
-  bool setTable(const char *table);
+  bool setTable(const string &table);
 
   /**
    * These are the query_log system variables.  So sysvar_enabled is
@@ -81,6 +82,21 @@
                            const string &scramble_bytes,
                            const string &client_password);
 
+  /**
+   * @brief
+   *   Escape a string for use as a single-quoted string value.
+   *
+   * @details
+   *   The string is escaped so that it can be used as a value in single quotes, like:
+   *   col='untrusted value'.  Therefore, double quotes are not escaped because they're
+   *   valid inside single-quoted values.  Escaping helps avoid SQL injections.
+   *
+   * @param[in] input Untrusted string
+   *
+   * @return Escaped string
+   */
+  string escapeString(const string &input);
+
   Session::shared_ptr _session; ///< Internal session for querying auth table
 };
 

=== modified file 'plugin/auth_schema/docs/index.rst'
--- plugin/auth_schema/docs/index.rst	2011-10-07 03:06:19 +0000
+++ plugin/auth_schema/docs/index.rst	2011-10-08 19:47:11 +0000
@@ -42,7 +42,8 @@
    :Default: ``auth.users``
    :Variable: :ref:`auth_schema_table <auth_schema_table>`
 
-   Schema-qualified table with ``user`` and ``password`` columns.
+   Schema-qualified table with ``user`` and ``password`` columns.  The auth table name
+   does not have to be quoted, and it cannot contain spaces (even if quoted).
 
 .. _auth_schema_variables:
 

=== modified file 'plugin/auth_schema/module.cc'
--- plugin/auth_schema/module.cc	2011-10-07 03:00:33 +0000
+++ plugin/auth_schema/module.cc	2011-10-08 19:47:11 +0000
@@ -43,7 +43,13 @@
 
 bool update_table(Session *, set_var *var)
 {
-  return auth_schema->setTable(var->value->str_value.ptr());
+  if (not var->value->str_value.data())
+  {
+    errmsg_printf(error::ERROR, _("auth_schema table cannot be NULL"));
+    return true; // error
+  }
+  const string table(var->value->str_value.data());
+  return auth_schema->setTable(table);
 }
 
 static void init_options(module::option_context &context)
@@ -60,7 +66,7 @@
   const module::option_map &vm= context.getOptions();
 
   if (not vm["table"].as<string>().empty())
-    auth_schema->setTable(vm["table"].as<string>().c_str());
+    auth_schema->setTable(vm["table"].as<string>());
 
   context.add(auth_schema);
 

=== modified file 'plugin/auth_schema/tests/r/basic.result'
--- plugin/auth_schema/tests/r/basic.result	2011-10-07 03:00:33 +0000
+++ plugin/auth_schema/tests/r/basic.result	2011-10-08 19:47:11 +0000
@@ -65,4 +65,20 @@
 SELECT 'auth re-enabled';
 auth re-enabled
 auth re-enabled
+connect(localhost,"'; drop table auth.users; select '",test_pass,test,MASTER_PORT,);
+ERROR 28000: Access denied for user '"'; drop table auth.users; select '"' (using password: YES)
+SHOW TABLES FROM auth;
+Tables_in_auth
+users
+users2
+SET GLOBAL auth_schema_table=NULL;
+ERROR HY000: Incorrect arguments to SET
+SHOW VARIABLES LIKE 'auth_schema_table';
+Variable_name	Value
+auth_schema_table	auth.users
+SET GLOBAL auth_schema_table='';
+ERROR HY000: Incorrect arguments to SET
+SHOW VARIABLES LIKE 'auth_schema_table';
+Variable_name	Value
+auth_schema_table	auth.users
 DROP SCHEMA auth;

=== modified file 'plugin/auth_schema/tests/t/basic.test'
--- plugin/auth_schema/tests/t/basic.test	2011-10-07 03:00:33 +0000
+++ plugin/auth_schema/tests/t/basic.test	2011-10-08 19:47:11 +0000
@@ -73,4 +73,22 @@
 connect(conn3, localhost, test_user, test_pass,,);
 SELECT 'auth re-enabled';
 
+# Escape user name; avoid SQL injection.
+--replace_result $MASTER_MYSOCK MASTER_SOCKET $MASTER_MYPORT MASTER_PORT
+--replace_regex /@'.*?'/@'LOCALHOST'/
+--error ER_ACCESS_DENIED_ERROR
+connect(conn4, localhost, "'; drop table auth.users; select '", test_pass,,);
+
+SHOW TABLES FROM auth;
+
+# Don't crash if we try to set the auth table to NULL.
+--error ER_WRONG_ARGUMENTS
+SET GLOBAL auth_schema_table=NULL;
+SHOW VARIABLES LIKE 'auth_schema_table';
+
+# And don't permit a blank string for the auth table.
+--error ER_WRONG_ARGUMENTS
+SET GLOBAL auth_schema_table='';
+SHOW VARIABLES LIKE 'auth_schema_table';
+
 DROP SCHEMA auth;

